"""
Client's models for validation.
"""


from enum import Enum
from typing import Optional

from pydantic.v1 import (
    BaseModel,
    conint,
    root_validator,
)

from ..logger import get_logger


logger = get_logger("models")


class BaseRequestModel(BaseModel):
    """
    Base model for parameters common in all requests.
    """

    pretty: Optional[bool]
    size: Optional[conint(ge=1, le=100)]


class AdditionalParametersModel(BaseModel):
    """
    Model for additional parameters which are shared across different APIs.
    """

    min_likelihood: Optional[conint(ge=1, le=10)]
    required: Optional[str]
    titlecase: Optional[bool]
    data_include: Optional[str]
    include_if_matched: Optional[bool]


class BaseSearchModel(BaseRequestModel):
    """
    Common fields validation model for search APIs (company, person).
    """

    query: Optional[dict]
    sql: Optional[str]
    from_: Optional[conint(ge=0, le=9999)]
    scroll_token: Optional[str]
    titlecase: Optional[bool]

    @root_validator(pre=True)
    def query_or_sql(cls, v):
        """
        Checks only one between 'query' and 'sql' is provided.
        """
        if not bool(v.get("query")) ^ bool(v.get("sql")):
            raise ValueError(
                "It is required to provide a value for either the 'query'"
                " parameter or the 'sql' parameter in order"
                " to receive a successful response."
                " See documentation @"
                " https://docs.peopledatalabs.com/docs/"
                "search-api#building-a-query ,"
                " https://docs.peopledatalabs.com/docs/"
                "company-search-api#building-a-query"
            )

        return v


class FieldEnum(str, Enum):
    """
    Valid values for 'field' parameter of autocomplete API.
    """

    company = "company"
    country = "country"
    industry = "industry"
    location = "location"
    major = "major"
    region = "region"
    role = "role"
    school = "school"
    sub_role = "sub_role"
    skill = "skill"
    title = "title"


class AutocompleteModel(BaseRequestModel):
    """
    Validator model for autocomplete API.
    """

    field: FieldEnum
    text: Optional[str]


class SkillModel(BaseRequestModel):
    """
    Validator model for skill API.
    """

    skill: str


class JobTitleModel(BaseRequestModel):
    """
    Validator model for job_title API.
    """

    job_title: str


class IPModel(BaseModel):
    """
    Validator model for ip API.
    """

    ip: str
    return_ip_metadata: Optional[bool]
    return_ip_location: Optional[bool]
    return_person: Optional[bool]
    return_if_unmatched: Optional[bool]
    pretty: Optional[bool]
    titlecase: Optional[bool]



"""
Defines all API endpoints for the 'School' section.
"""


from pydantic.v1.dataclasses import dataclass

from . import Endpoint
from ..logger import get_logger
from ..models import school as school_models


logger = get_logger("endpoints.school")


@dataclass
class School(Endpoint):
    """
    Class for all APIs of "school" type.
    """

    section: str = "school"

    def cleaner(self, **kwargs):
        """
        Calls PeopleDataLabs' school/clean API.

        Args:
            **kwargs: Parameters for the API as defined
                in the documentation.

        Returns:
            A requests.Response object with the result of the HTTP call.
        """
        return self._cleaner(school_models.CleanerModel, **kwargs)



"""
Logging utility module to invoke different children of the same root logger,
from different modules (and tests).
"""


import logging

from .settings import settings


logging.basicConfig(format=settings.log_format, style="{")


def get_logger(name: str = None, log_level: str = settings.log_level):
    """
    Utility function to retrieve a logger.

    Args:
        name (:obj:`str`, optional): Specify a name if you want
            to retrieve a logger which is a child of
            PeopleDataLabs logger.
        log_level (:obj:`str`, optional): Specify the log level
            for this particular logger.

    Returns:
        The PeopleDataLabs logger, or one of its children.
    """
    default = "PeopleDataLabs"
    if name:
        logger = logging.getLogger(f"{default}.{name}")
    else:
        logger = logging.getLogger(default)

    if log_level is not None:
        logger.setLevel(log_level)
    logger.debug("Got logger %s", logger)

    return logger



"""
A settings singleton to share settings across different modules of the package.

Settings also loads environment variables eventually declared in an .env
file.
"""

import os

from dotenv import find_dotenv, load_dotenv
from pydantic.v1 import HttpUrl
from pydantic.v1.dataclasses import dataclass


@dataclass
class Settings:
    """
    Singleton holding app's settings.

    Settings are eventually overridden if a .env file is provided, or
    environment variables are defined.

    All env variables should be in the form of PDL_<setting name>
    """

    api_key: str = None
    base_path: HttpUrl = "https://api.peopledatalabs.com/"
    log_level: str = None
    log_format: str = "{asctime} [{levelname}] - {name}.{funcName}: {message}"
    version: str = "v5"
    version_re: str = r"^v[0-9]$"
    sandbox_base_path: HttpUrl = "https://sandbox.api.peopledatalabs.com/"

    def __post_init__(self):
        load_dotenv(dotenv_path=find_dotenv(usecwd=True))
        for key in self.__dict__:
            env_key = "PDL_" + key.upper()
            self.__dict__[key] = os.getenv(env_key, self.__dict__[key])


settings = Settings()



"""
Models for input parameters of the School APIs.
"""


from typing import Optional

from pydantic.v1 import root_validator

from . import BaseRequestModel


class CleanerModel(BaseRequestModel):
    """
    Validation model for School 'cleaner' API.
    """

    name: Optional[str]
    website: Optional[str]
    profile: Optional[str]

    @root_validator(pre=True)
    def at_least_one(cls, value):
        """
        Checks that at least one parameter is valued.
        """
        if not any(value.values()):
            raise ValueError(
                "At least one between 'name' 'website' or 'profile' is"
                " required. See documentation @"
                " https://docs.peopledatalabs.com/docs/cleaner-apis#parameters"
            )

        return value



"""
Models for input parameters of the Person APIs.
"""


from enum import Enum
from typing import List, Optional, Union

from pydantic.v1 import (
    BaseModel,
    EmailStr,
    root_validator,
    conint,
    validator,
)

from . import (
    AdditionalParametersModel,
    BaseRequestModel,
    BaseSearchModel,
)


class PersonBaseModel(BaseModel):
    """
    Base parameters model for the enrichment and identify API.
    """

    birth_date: Optional[Union[List[str], str]]
    company: Optional[Union[List[str], str]]
    country: Optional[str]
    email: Optional[Union[List[EmailStr], EmailStr]]
    email_hash: Optional[Union[List[str], str]]
    first_name: Optional[Union[List[str], str]]
    last_name: Optional[Union[List[str], str]]
    lid: Optional[Union[List[str], str]]
    locality: Optional[str]
    location: Optional[Union[List[str], str]]
    middle_name: Optional[Union[List[str], str]]
    name: Optional[Union[List[str], str]]
    phone: Optional[Union[List[str], str]]
    pdl_id: Optional[Union[List[str], str]]
    postal_code: Optional[Union[List[str], str]]
    profile: Optional[Union[List[str], str]]
    region: Optional[str]
    school: Optional[Union[List[str], str]]
    street_address: Optional[str]
    pdl_id: Optional[str]
    min_likelihood: Optional[conint(ge=1, le=10)]
    required: Optional[str]
    data_include: Optional[str]
    include_if_matched: Optional[bool]

    @root_validator(pre=True)
    def at_least_one(cls, value):
        """
        Checks that at least one parameter is valued.
        """
        if not any(value.values()):
            raise ValueError(
                "'params' cannot be empty."
                " See documentation @"
                " https://docs.peopledatalabs.com/docs/enrichment-api"
            )

        return value


class EnrichmentModel(
    BaseRequestModel, PersonBaseModel, AdditionalParametersModel
):
    """
    Model for the enrichment API.
    """


class IdentifyModel(
    BaseRequestModel, PersonBaseModel, AdditionalParametersModel
):
    """
    Model for the identify API.

    The identify API uses same fields for parameters as the enrichment
    API, with the only difference that none of the fields accept
    multiple values.
    """

    @root_validator(pre=True)
    def no_lists(cls, v):
        """
        Checks none of the values are lists.
        """
        are_lists = [isinstance(field, list) for field in v.values()]
        if any(are_lists):
            raise ValueError(
                "Identify API does not take multiple values"
                " for parameters. See documentation @ "
                "https://docs.peopledatalabs.com/docs/"
                "identify-api-input-parameters"
            )

        return v


class PersonBulkParamsModel(BaseModel):
    """
    Model for the validation of the 'params' field in the person/bulk API.
    """

    metadata: Optional[dict]
    params: PersonBaseModel = ...


class BulkModel(BaseRequestModel, AdditionalParametersModel):
    """
    Model for the person/bulk API.
    """

    requests: List[PersonBulkParamsModel]

    @validator("requests", pre=True)
    def must_contain_params(cls, value):
        """
        Checks that 'requests' is not empty.
        """
        if not value:
            raise ValueError(
                "'requests' cannot be empty."
                " See documentation @"
                " https://docs.peopledatalabs.com/docs/bulk-enrichment-api"
            )

        return value


class DatasetEnum(str, Enum):
    """
    Valid values for 'dataset' field of search API.
    """

    resume = "resume"
    email = "email"
    phone = "phone"
    mobile_phone = "mobile_phone"
    street_address = "street_address"
    consumer_social = "consumer_social"
    developer = "developer"
    all = "all"


class SearchModel(BaseSearchModel):
    """
    Model for validation of person search API.
    """

    dataset: Optional[str]

    @validator("dataset", pre=True)
    def validate_datasets(cls, v):
        """
        Checks each passed dataset to be of the allowed ones.
        """
        res = []
        for dataset in [e.strip() for e in v.split(",") if e]:
            if dataset.startswith("-"):
                res.append("-" + DatasetEnum(dataset[1:]))
            else:
                res.append(DatasetEnum(dataset))

        return ",".join(res)
